#include <stdlib.h>
#include <stdio.h>
#include <sys/wait.h>        
#include <sys/types.h>       
#include <unistd.h>          
#include <errno.h>           
#include <string.h>
#include <linux/limits.h> // per PIPE_BUF, Linux specific !

// utility macro
#define SYSCALL(r,c,e) \
    if((r=c)==-1) { perror(e);exit(errno); }

#if !defined(MAXLINE)
#define MAXLINE 2024
#endif

void args2line(char *args[], char line[], const size_t len) {
    int i=0, p=0;
    while(args[i] != NULL) {
	if (p+strlen(args[i])+1 >= len) {
	    fprintf(stderr, "ERRORE: MAXLINE=%d troppo piccolo, aumentalo!\n",MAXLINE);
	    exit(EXIT_FAILURE);
	}
	strncpy(&line[p], args[i], strlen(args[i]));	
	p+=strlen(args[i]);
	line[p]=' ';
	++p, ++i;
    }
}

int main(int argc, char *argv[]) {
    if (argc == 1) {
	fprintf(stderr, "usa: %s comando [argomenti-comando]\n", argv[0]);
	return EXIT_FAILURE;
    }

    int r,pid;
    int channel[2];
    SYSCALL(r,pipe(channel), "pipe");
    SYSCALL(pid, fork(), "fork");

    if(pid == 0) {
	close(channel[0]);  	// chiudo il canale che non uso
	dup2(channel[1],1);     // duplico lo stdout su channel[1] (chiude 1)
	close(channel[1]);      // chiudo channel[1]
	
	execvp(argv[1],&argv[1]);
	perror("execvp");
	return errno;
    } 
    // chiudo il canale che non uso altrimenti la read su channel[0] non termina 
    // perche' c'e' ancora un descrittore in scrittura aperto !
    close(channel[1]);  
    
    char buffer[PIPE_BUF];
    char line[MAXLINE];
    args2line(argv, line, MAXLINE);
    fprintf(stdout, "*****************");
    for(int i=0;i<strlen(line);++i) fputs("*",stdout);
    fprintf(stdout, "\n");
    fprintf(stdout, "Comando eseguito: %s\n",line);
    fprintf(stdout, "*****************");
    for(int i=0;i<strlen(line);++i) fputs("*",stdout);
    fprintf(stdout, "\n");
    fprintf(stdout, "---------------------Risultato:\n");
    int n;
    while((n=read(channel[0], buffer, 1024))>0) {
	fprintf(stdout, "%s\n", buffer);
	memset(buffer, 0, 1024);
    }
    close(channel[0]);

    int status;
    SYSCALL(pid, wait(&status),"wait");
    fprintf(stdout, "---------------------------[%d]\n", WEXITSTATUS(status));
    return 0;
}





